@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 #
-my $revision = '$Id: Sanitizer.pm,v 1.90 2005/01/04 20:30:13 bre Exp $';
+my $revision = '$Id: Sanitizer.pm,v 1.94 2006/01/02 16:43:10 bre Exp $';
 my $version = 'Anomy 0.0.0 : Sanitizer.pm';
 #
 ##  Copyright (c) 2000-2005 Bjarni R. Einarsson. All rights reserved.
@@ -44,6 +44,7 @@ use strict;
 use Anomy::Log;
 use Anomy::MIMEStream;
 use Anomy::HTMLCleaner;
+use Anomy::Sanitizer::FileTypes qw( check_file_type );
 use IO::File;
 use Digest::MD5;
 use bytes;
@@ -51,7 +52,7 @@ use bytes;
 BEGIN {
     use Exporter  ();
     use vars      qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
-    $VERSION      = do { my @r = (q$Revision: 1.90 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
+    $VERSION      = do { my @r = (q$Revision: 1.94 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
     @ISA          = qw(Exporter);
     @EXPORT       = qw( );
     @EXPORT_OK    = qw( );
@@ -71,7 +72,7 @@ my $default_config =
     "feat_log_trace"  => 0,  # Attachment logs: Complete (all trace info).
 	"feat_log_after"  => 0,  # Scratch area sizes - see below. 0 = Off.
     "feat_files"      => 1,  # Enable filename-based policy decisions.
-	"feat_sane_names" => 1,  # Make sure file names match the content-type.
+	"feat_sane_names" => 1,  # Make sure names match content & mime-type.
 	"feat_mime_files" => 0,  # Always check the mime-type's default name too.
     "feat_force_name" => 0,  # Force all parts (except text/plain or
                              # text/html parts) to have file names.
@@ -333,7 +334,7 @@ my $default_config =
         "See the attached sanitization log for more details or contact your\012".
         "system administrator.\012\012".
         "The removed attachment's name was:\012\012".
-        "\t%FILENAME\012\012".
+        "\t%FILENAME (evaluated as %CHECKEDNAME)\012\012".
         "It might be a good idea to contact the sender and warn them that\012".
         "their system is infected.\012****\012",
 
@@ -344,7 +345,7 @@ my $default_config =
         "file name:\012\012".
         "\t%SAVEDNAME\012\012".
         "The removed attachment's original name was:\012\012".
-        "\t%FILENAME\012\012".
+        "\t%FILENAME (evaluated as %CHECKEDNAME)\012\012".
         "It is recommended that you contact your system administrator if you\012".
         "need access to the file.  It might also be a good idea to contact the\012".
         "sender, and warn them that their system may be infected.\012****\012",
@@ -653,6 +654,12 @@ sub configure
 
 		# Get a bunch of default filenames from /etc/mime.types
 		$self->load_mime_filenames($conf->{"system_mime_types"});
+
+        # Make sure a few config variables are safe for use within
+        # message headers.
+        $conf->{"msg_defanged"}    =~ s/[^0-9A-Z-]//gsi;
+        $conf->{"msg_blacklisted"} =~ s/[^0-9A-Z-]//gsi;
+        $conf->{"var_def_charset"} =~ s/[^0-9A-Z-]//gsi;
 	}
 
     return undef;
@@ -1292,9 +1299,23 @@ sub SanitizeFile
     
     # Make sure the list doesn't repeat itself.
 	my $l = undef;
-     @filenames = grep { ($_ ne $l) && ($l = $_) } sort @filenames;
+    @filenames = grep { ($_ ne $l) && ($l = $_) } sort @filenames;
     my %filenames = map { $_ => 1 } @filenames;
 
+    # Perform basic magic on the first few bytes of data and check the
+    # filetype integrity based on that.  Add names to the list of 
+    # filenames we check.
+    my $part_readahead = $part->Readahead(512);
+    my ($unused, $unused, $unused, @types) = 
+        check_file_type( mime_type  => $$typep, 
+                         snippet    => $part_readahead );
+    foreach my $t (@types)
+    {
+        my $name = "filetype.".$t->{id};
+        push @filenames, $name; 
+        $filenames{$name} = 1;
+    }
+
     # Look even harder...
 	if ($conf->{"feat_mime_files"})
 	{
@@ -1336,6 +1357,7 @@ sub SanitizeFile
 
     # Store original file name & type, initialize other variables.
     my $ofn = $$fnp;
+    my $attname = $$fnp;
     my $otype = $$typep;
     my $pol = undef;
     my @matched = ( );
@@ -1417,6 +1439,12 @@ sub SanitizeFile
 											$filename, $fh, $dig, $fnp);
                         $scanned = 1;
 
+                        if ($scan_result == 1) 
+                        {
+                            # File was disinfected, message will be modified 
+                            # one way or another.
+							$minc = 1 unless ($minc);
+                        }
 						if (my $n = $self->{"filescan-newfile"})
 						{
 						    print STDERR "*** New file! $n\n";
@@ -1504,7 +1532,8 @@ sub SanitizeFile
                 $fn =~ s/^.*\///g;
 
                 # Make info available to the replacement template code.
-	            $conf->{"var_filename"}    = $$fnp;
+	            $conf->{"var_checkedname"}    = $$fnp;
+	            $conf->{"var_filename"}     = $attname;
 				$conf->{"var_savedname"}   = $fn;
 				$conf->{"var_virusname"}   = $self->{"filescan-virusname"};
                 $conf->{"var_summary"}     = $self->{"filescan-summary"};
@@ -1572,20 +1601,40 @@ sub SanitizeFile
 	# Cleanup.
 	unlink $filename if ($filename);
 
-	# Ensure the file name matches the MIME type
-	my $check = undef;
-	if (($conf->{"feat_sane_names"}) &&
-        ($check = $self->{"name_type_checks"}->{lc($$typep)}) && 
-	    ($$fnp !~ $check))
-	{
-        $log->entry("mime-type_mismatch", SLOG_INFO, 
-		            { file => $$fnp, type => $$typep },
-					"File name doesn't match MIME type, defanging.");
+	# Ensure the file name matches the content and MIME type
+	if (($conf->{"feat_sane_names"}) && ((!$ofn) || ($$fnp eq $ofn)))
+    {
+	    my $check = undef;
+        my $bad_name = 0;
 
-		$$fnp =~ s/\./_/g;
-        $$fnp .= ".". $conf->{"msg_defanged"} ."-".$self->{"mod_id"};
-        $$typep = "application/". $conf->{"msg_defanged"} ."-".$self->{"mod_id"};
-	}
+        my ($is_evil, $is_suspicious) = 
+            check_file_type( mime_type  => $$typep, 
+                             file_names => [ $$fnp ],
+                             snippet    => $part_readahead );
+
+        if ($is_evil || $is_suspicious)
+        {
+            $log->entry("content_mismatch", SLOG_INFO, 
+		                { file => $$fnp, type => $$typep },
+					    "File name doesn't match file contents, defanging.");
+            $bad_name = 1;
+        }
+        elsif (($check = $self->{"name_type_checks"}->{lc($$typep)}) && 
+	           ($$fnp !~ $check))
+        {
+            $log->entry("mime-type_mismatch", SLOG_INFO, 
+		                { file => $$fnp, type => $$typep },
+					    "File name doesn't match MIME type, defanging.");
+            $bad_name = 1;
+        }
+
+        if ($bad_name)
+        {
+		    $$fnp =~ s/\./_/g;
+            $$fnp .= ".". $conf->{"msg_defanged"} ."-".$self->{"mod_id"};
+            $$typep = "application/". $conf->{"msg_defanged"} ."-".$self->{"mod_id"};
+        }
+    }
 
     # Remove spooky characters from file name.
 	if (my $clist = $conf->{"file_characters"})
@@ -1729,7 +1778,7 @@ sub SanitizeHeaders
         }
         # This is more strict than the following header scan, and is 
         # designed to explicitly defang the Outlook Date: overflow.
-        if ($writer->{"rawheader"} =~ s/(Date:\s*[^\012]{60,60})([^\012]+)/$1\012X-$conf->{"msg_defanged"}-Date: [$self->{"mod_id"}] $2/gs)
+        if ($writer->{"rawheader"} =~ s/(\nDate:\s*[^\012]{65,65})([^\012]+)/$1\012X-$conf->{"msg_defanged"}-Date: [$self->{"mod_id"}] $2/gs)
         {
             $log->entry("split-date", SLOG_WARNING|SLOG_INFO, 
 			            { id => $self->{"mod_id"} },
@@ -1737,14 +1786,14 @@ sub SanitizeHeaders
 			$self->{"mod_id"}++;
         }
         # This will limit the length of each individual word in the headers
-        # to 196 characters, inserting a text marker and spaces when longer
+        # to 256 characters, inserting a text marker and spaces when longer
         # words are encountered.  This is designed to foil attacks based on 
         # vulnerabilities like those described in various bugtraq posts 
         # in July 2000, including the USSR-2000050 advisory.
         #
         my $spaces = " " x (1 + rand()*7);
         $spaces = " " if ($conf->{"feat_testing"});
-        if ($writer->{"rawheader"} =~ s/(\S{196,196})/$conf->{"msg_defanged"}\[$self->{"mod_id"}]:$1$2 "$spaces" /gs)
+        if ($writer->{"rawheader"} =~ s/(\S{256,256})/$conf->{"msg_defanged"}\[$self->{"mod_id"}]:$1$2 "$spaces" /gs)
         {
             $log->entry("split-words", SLOG_WARNING|SLOG_INFO, 
 			            { id => $self->{"mod_id"} },
