@@ -105,7 +105,7 @@ for a in *.t; do
 	    echo installed
             
         else
-	    if diff -u -b test.out "$TEST_RESULTS/$test.ok" >test.diff; then
+	    if diff -u -b "$TEST_RESULTS/$test.ok" test.out >test.diff; then
 	        T="$TEST_RESULTS/$test"
 	        rm -f test.* $T.out $T.log $T.diff
 	        echo ok
@@ -114,7 +114,7 @@ for a in *.t; do
 		    mv -f $t "$TEST_RESULTS"/`echo $t |sed -e "s/^test/$test/"`
 		done
 	        echo "failed  (moved result files to $TEST_RESULTS)"
-		FAILED=1
+	 	let FAILED=$FAILED+1
 	    fi
         fi
 	[ -e "$TEST_RESULTS/$test.ok.rot13" ] && rm -f "$TEST_RESULTS/$test.ok" 
